

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include"subs.h"
#include"dsyevc3.h"
#include"dsyevj3.h"

int dsvd(float a[3][3], int m, int n, float *w, float v[3][3]);

void printpart_nei(particle *snap,  int n_nei, int *nei_list)
{
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      printf("%.2f %.2f %.2f\n",snap[ni].x, snap[ni].y, snap[ni].z);
   }
}




// multiply two matrices together returning a third matrix
// note a_ij   a_11  a12 a13 is first row convention
// C = A*B
void multmatrix(float C[3][3], float A[3][3], float B[3][3])
{
   for(int i=0;i<3;i++){
      for(int j=0;j<3;j++){
         float sum=0.0;
          for(int k=0;k<3;k++) sum += A[i][k]*B[k][j];
         // for(int k=0;k<3;k++) sum += A[k][i]*B[j][k];
         C[i][j] = sum;
      }
   }
}

// A->B
void copymatrix(float B[3][3], float A[3][3])
{
   for(int i=0;i<3;i++){
      for(int j=0;j<3;j++) B[i][j] = A[i][j];
   }
}
// in place transpose
void transpose_matrix(float A[3][3])
{
   float C[3][3];
   copymatrix(C,A);
   for(int i=0;i<3;i++){
      for(int j=0;j<3;j++) A[i][j] = C[j][i];
   }
}

// determinent of a 3x3 matrix
float det(float A[3][3])
{
   float sum = 
     A[0][0]*A[1][1]*A[2][2] + A[0][1]*A[1][2]*A[2][0] + A[0][2]*A[1][0]*A[2][1]
    -A[0][2]*A[1][1]*A[2][0] - A[0][1]*A[1][0]*A[2][2] - A[0][0]*A[1][2]*A[2][1];
   return sum;
}

// rotate particle positions using rotation matrix Q
// matrix convention A_ij   i gives row j gives column
// all particles are rotated
void rotate_matrix(particle *snap_pos,  float Q[3][3], int np){
   for(int i=0;i<np;i++){
      float xtemp       = Q[0][0]*snap_pos[i].x   // might need to reverse or flip indices
                        + Q[0][1]*snap_pos[i].y 
                        + Q[0][2]*snap_pos[i].z;
      float ytemp       = Q[1][0]*snap_pos[i].x 
                        + Q[1][1]*snap_pos[i].y 
                        + Q[1][2]*snap_pos[i].z;
      float ztemp       = Q[2][0]*snap_pos[i].x 
                        + Q[2][1]*snap_pos[i].y 
                        + Q[2][2]*snap_pos[i].z;
      snap_pos[i].x = xtemp;
      snap_pos[i].y = ytemp;
      snap_pos[i].z = ztemp;
   }
}

void printmatrix(const char string[], float A[3][3])
{
   printf("%s[ %.4f %.4f %.4f ]\n",string,A[0][0],A[0][1],A[0][2]);
   printf("%s[ %.4f %.4f %.4f ]\n",string,A[1][0],A[1][1],A[1][2]);
   printf("%s[ %.4f %.4f %.4f ]\n",string,A[2][0],A[2][1],A[2][2]);
}

///////////////////////////////////////////////////////////////////////
// rotate an entire solid body  
// you are given two sets of points
// rotate the second set to be consistent with the first set
// following:   http://en.wikipedia.org/wiki/Wahba%27s_problem
// http://en.wikipedia.org/wiki/Singular_value_decomposition
// http://www.ltu.se/cms_fs/1.51590!/svd-fitting.pdf
// here the particles in snap_pos_sec is rotated and com corrected
// the particles in snap_pos_init are com corrected
// last particle is not included in correlations calculation but is rotated
// #define M2FIRST 0   // is 1 if second body is first particle rather than last
// defined in subs.h
void rotate_body(particle *snap_pos_init, particle *snap_pos_sec, int nparticles)
{
   // commented out because done outside routine!
   // com_sub(snap_pos_init, nparticles);  // subtract out the center of masses
   // com_sub(snap_pos_sec, nparticles);
   float B[3][3]; // for the correlation matrix
   for(int i=0;i<3;i++){
     for(int j=0;j<3;j++) B[i][j] = 0.0;
   }
   int ifirst = 0;
   int ilast = nparticles-1;
   if (M2FIRST==1){
      ifirst=1;
      ilast=nparticles;
   }
   
   for(int i=ifirst;i<ilast;i++){ // don't include last particle 
      B[0][0] += snap_pos_init[i].x * snap_pos_sec[i].x;
      B[1][1] += snap_pos_init[i].y * snap_pos_sec[i].y;
      B[2][2] += snap_pos_init[i].z * snap_pos_sec[i].z;
      B[0][1] += snap_pos_init[i].x * snap_pos_sec[i].y;
      B[1][0] += snap_pos_init[i].y * snap_pos_sec[i].x;
      B[0][2] += snap_pos_init[i].x * snap_pos_sec[i].z;
      B[2][0] += snap_pos_init[i].z * snap_pos_sec[i].x;
      B[1][2] += snap_pos_init[i].y * snap_pos_sec[i].z;
      B[2][1] += snap_pos_init[i].z * snap_pos_sec[i].y;
   }
   float U[3][3]; // rotation matrix 
   float S[3];    // diagonals 
   float V[3][3];  // rotation matrix  
   float M[3][3];  // for new diagonal matrix
   float Smat[3][3];  // Smatrix for testing
   for(int i=0;i<3;i++){ // just in case
     S[i] = 0.0;
     for(int j=0;j<3;j++){
          V[i][j] = 0.0; M[i][j] = 0.0; U[i][j] = 0.0;   Smat[i][j]=0.0;
     }
   }

/*
      B[0][0] = 1.0; B[0][1] = 0.5; B[0][2] = 0.3;
      B[1][0] = 1.0; B[1][1] = 1.0; B[1][2] = 0.0;
      B[2][0] = 0.0; B[2][1] = 0.0; B[2][2] = 1.0;
*/

   // printmatrix("B",B); 
   // printf("detB =%.2f\n",det(B));

   // singular value decomposition
// returned is v not vtranspose!
// U and V are orthogonal not special orthogonal so they are not rotations
// but each row is perpendicular to each row
// their determinants are not 1
   int jj = dsvd(B, 3, 3, S, V);  // B becomes U
   copymatrix(U,B);
   // printmatrix("U",U); 
   // printmatrix("V",V); 
   // printf("detU =%.2f\n",det(U));
   // printf("detV =%.2f\n",det(V));
   // printf("diags S[%.2f %.2f %.2f]\n",S[0],S[1],S[2]);

   transpose_matrix(V); // transpose V
   // printf("detVt =%.2f\n",det(V));
   float tempM[3][3];  // temporary matrix
   float R[3][3]; // rotation matrix

/*
   //  B = U*S*Vt, this check worked!
   Smat[0][0] = S[0]; Smat[1][1] = S[1]; Smat[2][2] = S[2];
   printmatrix("Smat",Smat); 
   multmatrix(tempM,Smat,V); // check svd
   multmatrix(R,U,tempM);
   printmatrix("B",R); 
*/

 // fill up diagonal matrix
   M[0][0] = 1.0; M[1][1] = 1.0; M[2][2] = det(U)*det(V);
   // printmatrix("M",M); 

   // R =  U*M*Vt
   multmatrix(tempM,M,V);
   multmatrix(R,U,tempM);
   // printmatrix("R",R);  // this should be the rotation matrix

   // now do the rotation, all particles are rotated
   rotate_matrix(snap_pos_sec,  R, nparticles);
}

/////////////////////////////////////////////////////
// compute tensor of <x_i x_j> where x_i = x_i-xmean_i tensor
// return eigenvalues 
// return an angle that we can plot for the strain direction
// the problem with this routine is that it projects onto spherical
// coordinates so the strains are not good
// the angle is fine though
void nei_tensor_2d(particle *snap_pos, int n_nei, int *nei_list,
             double *lambda1, double *lambda2, double *angle)
{
   double xsum = 0.0; double ysum = 0.0; double zsum = 0.0;
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      xsum += snap_pos[ni].x;
      ysum += snap_pos[ni].y;
      zsum += snap_pos[ni].z;
   }
   double3 mean; // mean of neighbhorhood
   mean.x = xsum/(double)n_nei;  mean.y = ysum/(double)n_nei; mean.z = zsum/(double)n_nei;

   double r = sqrt(mean.x*mean.x + mean.y*mean.y+ mean.z*mean.z);
   double phi = atan2(mean.y,mean.x); // spherical coords  theta=[-pi/2,pi/2]
   double theta = asin(mean.z/r);   // z=1 for theta=pi/2, theta=0 on equator

   // unit vectors
   double3 ephi,etheta; // unit vectors on surface of sphere
   ephi.x =  sin(phi); ephi.y =  -cos(phi); ephi.z = 0.0;
   etheta.x = -sin(theta)*cos(phi); etheta.y = -sin(theta)*sin(phi); etheta.z =  cos(theta);
    // points up on equator

   double xxsum = 0.0; double xysum = 0.0; double yysum = 0.0;
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      double3 diff;
      diff.x = snap_pos[ni].x - mean.x;
      diff.y = snap_pos[ni].y - mean.y;
      diff.z = snap_pos[ni].z - mean.z;
   // using unit vectors on sphere add up the correlation tensor
      double diff_phi  = dotproduct(diff,ephi); // get components in phi  direction
      double diff_theta  = dotproduct(diff,etheta);  // get components in theta direction
      xxsum += diff_phi*diff_phi;  // x direction is e_phi
      yysum += diff_theta*diff_theta; 
      xysum += diff_phi*diff_theta;  // y direction is e_theta
   }
   // normalize
   xxsum/=(double)n_nei;  xysum/=(double)n_nei;  yysum/=(double)n_nei;  
   // get eigenvalues and angle for the 2x2 symmetric tensor
   double lam1,lam2,ang;
   eigen2b2(xxsum,yysum,xysum, &lam1, &lam2,&ang);
   *lambda1 = lam1; 
   *lambda2 = lam2; 
   *angle = ang; 
}

double dotproduct(double3 a, double3 b)
{
   return  a.x*b.x + a.y*b.y + a.z*b.z;
}

// find eigenvalues and eigenvectors for a 2x2 symmetric matrix
// the matrix is A with three components  Axx,Ayy,Axy
// the eigenvalues are lambda1, lambda2
// the eigenvectors are returns as a single angle theta where the eigenvector is  
//            cos(theta),sin(theta)
// there is no reason to return a second angle as the two eigenvectors must be perpendicular
// equation for eigenvalues satisfies
// lambda^2 -  lambda tr A + det A = 0
void eigen2b2(double Axx, double Ayy, double Axy, 
   double *lambda1, double *lambda2, double *theta)
{
   double tr_A = Axx +  Ayy; // trace
   double det_A = Axx*Ayy - Axy*Axy;  // determinant
   double ss =sqrt(tr_A*tr_A - 4.0*det_A);
   double lam1 = 0.5*(tr_A + ss); // solve quadratic equation 
   double lam2 = 0.5*(tr_A - ss);
   double ratio1  = (lam1 - Axx)/Axy; // this is y_1/x_1 for the eigenvector 1
   //    double ratio2  = (lam2 - Axx)/Axy; // dito for second eigenvector
   double ang1 = atan2(ratio1,1.0); // in radians
 
   *lambda1 = lam1; 
   *lambda2 = lam2; 
   *theta= ang1;
}

/////////////////////////////////////////
// used in sorting
int cmp (const void *a, const void *b) {
   if (*(double *)a < *(double *)b) return -1;
   if (*(double *)a > *(double *)b) return 1;
   return 0;
}

/////////////////////////////////////////////////////
// compute tensor of <x_i x_j> where x_i = x_i-xmean_i tensor
// in 3D
// return rank ordered sqrt of eigenvalues in sw
// order is small to large 0-2
// calls dsyevc3() and qsort(), needs cmp() routine
// problem with this routine is that it does not return a direction
// maximal eigenvalue is probably enough to get the maximal strain of
// if I projected the eigenvector for the maximal eigenvalue into
// spherical coordinates I would get a direction for the maximal strain value
// the routine dsyevc3(A, w); (from Kopp) does not return eigenvectors
// the routine dsyevj3(A, Q, w) does give eigenvectors!
int nei_tensor(particle *snap_pos, int n_nei, int *nei_list, double *sw)
{
   double xsum = 0.0; double ysum = 0.0; double zsum = 0.0;
   double x2sum = 0.0; double y2sum = 0.0; double z2sum = 0.0;
   double xysum = 0.0; double yzsum = 0.0; double xzsum = 0.0;
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      xsum += snap_pos[ni].x;
      ysum += snap_pos[ni].y;
      zsum += snap_pos[ni].z;
   }
   double xmean = xsum/(float)n_nei;  // mean of neighbhorhood
   double ymean = ysum/(float)n_nei;
   double zmean = zsum/(float)n_nei;
   // double rmean = sqrt(xmean*xmean + ymean*ymean + zmean*zmean);
   // printf("mean of nei %.3f %.3f %.3f\n",xmean,ymean,zmean);
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      x2sum += pow((snap_pos[ni].x - xmean),2.0);
      y2sum += pow((snap_pos[ni].y - ymean),2.0);
      z2sum += pow((snap_pos[ni].z - zmean),2.0);
      xysum += (snap_pos[ni].x - xmean)*(snap_pos[ni].y - ymean);
      xzsum += (snap_pos[ni].x - xmean)*(snap_pos[ni].z - zmean);
      yzsum += (snap_pos[ni].y - ymean)*(snap_pos[ni].z - zmean);
   }
   x2sum /= (float)n_nei;  // normalize
   y2sum /= (float)n_nei;  
   z2sum /= (float)n_nei;  
   xysum /= (float)n_nei;  
   xzsum /= (float)n_nei;  
   yzsum /= (float)n_nei;  
   
   double A[3][3]; // moment tensor
   A[0][0] = x2sum;
   A[1][1] = y2sum;
   A[2][2] = z2sum;
   A[0][1] = xysum; A[1][0] = xysum;
   A[0][2] = xzsum; A[2][0] = xzsum;
   A[1][2] = yzsum; A[2][1] = yzsum;
   double Q[3][3]; // eigenvector 
   // printf("A diag tensor %.4f %.4f %.4f\n",x2sum,y2sum,z2sum);
   // printf("A offd tensor %.4f %.4f %.4f\n",xysum,xzsum,yzsum);
   double w[3];
   int ss = dsyevc3(A, w);
   int ss2 = dsyevj3(A, Q, w); // Q should now contain eigenvectors
   // printf("w = %.4f %.4f %.4f \n", w[0], w[1], w[2]);
   qsort(w,3,sizeof(w[0]),cmp);
   // sw[0] = fmax((fmax(w[0],w[1])),w[2]);
   // sw[2] = fmin((fmin(w[0],w[1])),w[2]);
   // printf("w = %.4f %.4f %.4f \n", w[0], w[1], w[2]);
   sw[0] = sqrt(w[0]);
   sw[1] = sqrt(w[1]);
   sw[2] = sqrt(w[2]);
   printf("w = %.4f %.4f %.4f \n", sw[0], sw[1], sw[2]);
}

// return mean radius of the neighborlist
double r_mean(particle *snap_pos, int n_nei, int *nei_list) 
{
   double rm=0.0;
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      double radius = pow(snap_pos[ni].x ,2.0)
                    + pow(snap_pos[ni].y ,2.0) 
                    + pow(snap_pos[ni].z ,2.0); 
      radius = sqrt(radius);
      rm += radius;
   }
   return rm/(double)n_nei;
}



/////////////////////////////////////////
// some subroutines for particles and io
////////////////////////////////////////////////////
// return a list of all particles within a distance d0 of position x0,y0,z0
// and outside a radius from origin of rmin
int num_nei(particle *snap_pos, int np, float d0, float rmin,
   float x0, float y0, float z0, int *nei_list)
{
   int ncount = 0;
   for(int i=0;i<np;i++){
     double dd = sqrt( pow(snap_pos[i].x-x0,2.0)
                      +pow(snap_pos[i].y-y0,2.0)
                      +pow(snap_pos[i].z-z0,2.0));
     double rad = sqrt(pow(snap_pos[i].x,2.0)
                      +pow(snap_pos[i].y,2.0)
                      +pow(snap_pos[i].z,2.0));
     if ((dd<d0)&&(rad > rmin)){
       nei_list[ncount] = i;
       ncount++;
     }
   }
   return ncount;
}

// return the id of the nearest particle to position
int nei1(particle *snap_pos, int np, float x0, float y0, float z0)
{
   int nearest = 0;
   double dmin = 100.0;
   for(int i=0;i<np;i++){
     double dd = sqrt( pow(snap_pos[i].x-x0,2.0)
                      +pow(snap_pos[i].y-y0,2.0)
                      +pow(snap_pos[i].z-z0,2.0));
     if (dd < dmin){
        nearest = i;
        dmin = dd;
     }
   }
   return nearest;
}

// return distance between two particles with id1, id2 
double distance_two(particle *snap_pos, int id1, int id2)
{
   double dd = sqrt( pow(snap_pos[id1].x - snap_pos[id2].x,2.0)
                    +pow(snap_pos[id1].y - snap_pos[id2].y,2.0)
                    +pow(snap_pos[id1].z - snap_pos[id2].z,2.0));
   return dd;
}


// find the particle in neighbor list that has maximum  dl/l
// particle must be at least rmin away from the central particle
// that has index ncenter
// return strain and angle dtheta/dphi of orientation vector
void maxdl(particle *snap_pos_init, particle *snap_pos_t, 
       int n_nei, int *nei_list, double rmin, int ncenter, int posneg,
       double *strain, double *angle) 
{
   double strain_max = 0.0;
   int nimax=-1;
   for(int i=0;i<n_nei;i++){
      int ni = nei_list[i];
      double dist0 = distance_two(snap_pos_init, ncenter, ni);
      if (dist0 > rmin){
        double dist1 = distance_two(snap_pos_t, ncenter, ni);
        double dl_l = (dist1-dist0)/dist0;
        if (posneg==1){
          if (dl_l > strain_max){
             strain_max = dl_l;
             nimax = ni;
          }
        }
        else {
          if (dl_l < strain_max){
             strain_max = dl_l;
             nimax = ni;
          }
        }
      }
   }
   *strain = strain_max;
   double ang = maxstrainang(snap_pos_t, ncenter, nimax);
   *angle = ang; // angle of max strain in dtheta/dphi
}

// get polar angle between the two particles with indexes in1,in2
double maxstrainang(particle *snap_pos, int in1, int in2){
   double phi1 = atan2(snap_pos[in1].y, snap_pos[in1].x); 
   double phi2 = atan2(snap_pos[in2].y, snap_pos[in2].x); 
   double dphi = phi2-phi1;
   if (dphi > M_PI) dphi -= 2.0*M_PI;
   if (dphi < -M_PI) dphi += 2.0*M_PI;
   double r1 = sqrt( pow(snap_pos[in1].x,2.0)
                    +pow(snap_pos[in1].y,2.0)
                    +pow(snap_pos[in1].z,2.0));
   double r2 = sqrt( pow(snap_pos[in2].x,2.0)
                    +pow(snap_pos[in2].y,2.0)
                    +pow(snap_pos[in2].z,2.0));
   double theta1 = asin(snap_pos[in1].z/r1);
   double theta2 = asin(snap_pos[in2].z/r2);
   double dtheta = theta2-theta1;
   if (dtheta > M_PI/2.0) dtheta -= M_PI;
   if (dtheta < -M_PI/2.0) dtheta += M_PI;
   double angle = atan2(dtheta,dphi);
   return angle;
}


////////////////////////////////////////////////////////////
// compute position of center of mass and subtract it from 
// all particle positions
// depending upon M2FIRST either first or last particle is not used
void com_sub(particle *snap_pos, int np){
   double xsum=0.0; double ysum=0.0; double zsum=0.0; double msum=0.0;
   double vxsum=0.0; double vysum=0.0; double vzsum=0.0; 
   int ifirst = 0;
   int ilast = np-1;
   if (M2FIRST==1){
      ifirst=1;
      ilast=np;
   }
   for(int i=ifirst;i<ilast;i++){
      xsum += snap_pos[i].m*snap_pos[i].x; // mass weighted sums
      ysum += snap_pos[i].m*snap_pos[i].y;
      zsum += snap_pos[i].m*snap_pos[i].z;
      msum += snap_pos[i].m;  // sum of masses
      vxsum += snap_pos[i].m*snap_pos[i].vx; // mass weighted sums
      vysum += snap_pos[i].m*snap_pos[i].vy;
      vzsum += snap_pos[i].m*snap_pos[i].vz;
   }
   xsum /= msum; ysum /= msum; zsum /= msum; // these are com positions
   vxsum /= msum; vysum /= msum; vzsum /= msum; 
   for(int i=ifirst;i<ilast;i++){
     snap_pos[i].x -= xsum;
     snap_pos[i].y -= ysum;
     snap_pos[i].z -= zsum;
     snap_pos[i].vx -= vxsum;
     snap_pos[i].vy -= vysum;
     snap_pos[i].vz -= vzsum;
   }
}

////////////////////////////////////////////////////////////
// io notes
// His particle numbers start with 1 but I start with 0 indexing
/* structure of his record
   int i;
   float x,y,z,vx,vy,vz,ax,ay,az,mass,radius,pressure,density,energy;
   fread(&i,sizeof(x),1,fpi); // index
   fread(&x,sizeof(x),1,fpi); fread(&y,sizeof(x),1,fpi); fread(&z,sizeof(x),1,fpi);
   fread(&vx,sizeof(x),1,fpi); fread(&vy,sizeof(x),1,fpi); fread(&vz,sizeof(x),1,fpi);
   fread(&ax,sizeof(x),1,fpi); fread(&ay,sizeof(x),1,fpi); fread(&az,sizeof(x),1,fpi);
   fread(&mass,sizeof(x),1,fpi); fread(&radius,sizeof(x),1,fpi); 
   fread(&pressure,sizeof(x),1,fpi); fread(&density,sizeof(x),1,fpi);
   fread(&energy,sizeof(x),1,fpi); 
*/


////////////////////////////////////////////////////////////
// read in positions from a single snap 
// read in the nth snap with first one n=0
// ns is snap number
// np is number of particles
// don't save header info 
// open and closes entire file
float readpossnap(char* filename,int np, int ns,  particle *snap_pos)
{
   FILE *fpi;
   fpi=fopen(filename,"r"); 
   for (int j=0;j<ns;j++){
     float time = readsinglet(fpi,np); // advance file to desired snap 
   } 
   char startheader[180];  // for storing header stuff
   char headerinfo[180];
   char endheader[180];
   char startdata[180];
   char enddata[180];
   char kch[3];
   // readin the snap
   float time = readsinglet_pos(fpi,np, snap_pos, 
      startheader, headerinfo, endheader, startdata, enddata, kch);
   fclose(fpi);
   return time;
}

////////////////////////////////////////////////////////////
// get the number of snaps in file and dt and tmax
int getnsnaps(char* filename,int np, float *dt_ret, float *tmax_ret)
{
   FILE *fpi;
   fpi=fopen(filename,"r"); 
   float time=0.0;
   int jj= feof(fpi);  // is not zero if at end of file 
   int nsnaps=0;
   float tmax=0.0;
   while (jj ==0.0){
     time = readsinglet(fpi,np); // printf("%d %f\n",nsnaps,time);
     jj = feof(fpi); // printf("%d\n",jj);
     if (time >=0) nsnaps++;
     if (time > 0) tmax = time;
   }
   fclose(fpi);
   double dt =  tmax/(nsnaps-1);
   // printf("number of snaps = %d maxtime =%f dt =%.2f\n",nsnaps,tmax,dt);
   *dt_ret = dt;
   *tmax_ret = tmax;
   return nsnaps;
}

////////////////////////////////////////////////////////////
// reading a single data line here, just to advance file
void readsingle(FILE *fp)
{
   float xx[14];
   int i;
   fread(&i,sizeof(xx[0]),1,fp);
   fread(xx,sizeof(xx[0]),14,fp); // single extra character where?
// printf("%d\n",i);
   int j=0;
/*
   printf(" %f %f %f ",xx[j++],xx[j++],xx[j++]);
   printf(" %f %f %f ",xx[j++],xx[j++],xx[j++]);
   printf(" %f %f %f ",xx[j++],xx[j++],xx[j++]);
   printf(" %f %f %f ",xx[j++],xx[j++],xx[j++]);
   printf(" %f %f \n",xx[j++],xx[j++]);
*/
   char k[3];
   fread(k,sizeof(k[0]),1,fp); // single extra character here
}

////////////////////////////////////////////////////////////
// get the number of particles and return it
int findnp(char *filename)
{
   FILE *fpi;
   fpi=fopen(filename,"r"); 
   char string[180];
   fgets(string,180,fpi);
   // printf("%s",string);

   fgets(string,180,fpi);
   int np=0; // number of particles = nRows
   int nc=0;
   double t=0.0;
   sscanf(string+54,"%d, nColumns=%d, time=%lf",&np,&nc,&t); 
   // printf("np=%d nc=%d t=%f\n",np,nc,t);
   // printf("%s",string);

//FileType=XcParticles, isAscii=f, isBigEndian=f, nRows=5945, nColumns=15, time=0.0000e+00, sph=t;
   fclose(fpi);
   return np;
}

////////////////////////////////////////////////////////////
// read in a timestep record and return its time
// assume we are given filepointer at beginning of record
// advance to end of record
// don't do anything with data
float readsinglet(FILE *fpi,int np)
{
   char string[180];
   int npa=0; // number of particles = nRows
   int nc=0;
   double t=0.0;
   fgets(string,180,fpi); // start header
   fgets(string,180,fpi);  // header info
   sscanf(string+54,"%d, nColumns=%d, time=%lf",&npa,&nc,&t); 
   fgets(string,180,fpi);  // end header
// here starts the data record
   fgets(string,180,fpi); // start data record
   for(int i=0;i<np;i++) readsingle(fpi); // data
   fgets(string,180,fpi); // ends the record
   int ff = feof(fpi);
   if (ff==0) return t;
   else return -1.0;  // returns -1 if end of file has been reached
}

////////////////////////////////////////////////////////////
// read in a timestep record and fill particle structure array
// assume we are given filepointer at beginning of record
// advance to end of record
// particle position structure array filled with data
// also fill and return header strings
float readsinglet_pos(FILE *fpi,int np, particle *pos, 
     char* startheader, char* headerinfo, char* endheader, char* startdata,
     char* enddata, char* kch)
{
   char string[180];
   int npa=0; // number of particles = nRows
   int nc=0;
   double t=0.0;
   fgets(startheader,180,fpi);  // start header
   fgets(headerinfo,180,fpi);  // header info
   sscanf(headerinfo+54,"%d, nColumns=%d, time=%lf",&npa,&nc,&t); 
   fgets(endheader,180,fpi);  // end header
   fgets(startdata,180,fpi);  // here starts the data record
   float xx[14];
   int j;
   char k[3];
   for(int i=0;i<np;i++){
     fread(&j,sizeof(xx[0]),1,fpi); // particle number
     // printf("j=%d\n",j);
     fread(xx,sizeof(xx[0]),14,fpi); // particle data
     fread(k,sizeof(k[0]),1,fpi); // single extra character here
     pos[i].x =xx[0]; pos[i].y  =xx[1]; pos[i].z = xx[2];// particle position
     pos[i].vx=xx[3]; pos[i].vy =xx[4]; pos[i].vz =xx[5];// velocity 
     pos[i].ax=xx[6]; pos[i].ay =xx[7]; pos[i].az =xx[8];// acceleration
     pos[i].m=xx[9]; // mass
     pos[i].r=xx[10]; // radius
     pos[i].pressure = xx[11];
     pos[i].density  = xx[12]; 
     pos[i].energy   = xx[13];
   }
   kch[0] = k[0]; // store one of these, assume all the same
   fgets(enddata,180,fpi); // ends the record
   int ff = feof(fpi); // end of file?
   if (ff==0) return t;
   else return -1.0;  // returns -1 if end of file has been reached
}

////////////////////////////////////////////////////////////
// write a timestep record and using a given particle structure array
// assume we are given filepointer at beginning of record
// advance to end of record
float writesinglet_pos(FILE *fpo,int np, particle *pos, 
     char* startheader, char* headerinfo, char* endheader, char* startdata,
     char* enddata, char* kch)
{
   double t=0.0;
   fprintf(fpo,"%s",startheader,fpo);  // start header
   fprintf(fpo,"%s",headerinfo);  // header info
   fprintf(fpo,"%s",endheader);  // end header
   fprintf(fpo,"%s",startdata);  // here starts the data record
   float xx[14];
   int j;
   for(int i=0;i<np;i++){
     j=i+1; // yes this is correct
     fwrite(&j,sizeof(xx[0]),1,fpo); // particle number

     xx[0] = pos[i].x ; xx[1] = pos[i].y ; xx[2] = pos[i].z ;// particle position
     xx[3] = pos[i].vx; xx[4] = pos[i].vy; xx[5] = pos[i].vz;// velocity 
     xx[6] = pos[i].ax; xx[7] = pos[i].ay; xx[8] = pos[i].az;// acceleration
     xx[9] = pos[i].m; // mass
     xx[10] =pos[i].r; // radius
     xx[11] =pos[i].pressure;
     xx[12] =pos[i].density; 
     xx[13] =pos[i].energy;
     fwrite(xx,sizeof(xx[0]),14,fpo);  // particle data 
     fwrite(kch,sizeof(kch[0]),1,fpo); // single extra character here
   }
   fprintf(fpo,"%s",enddata); // ends the record
   int ff = feof(fpo); // end of file?
   if (ff==0) return t;
   else return -1.0;  // returns -1 if end of file has been reached
}


////////////////////////////////////////////////////////////
// assume we are given filepointer at beginning of record
// from two files, one we are writing and one we are modifying from
// advance to end of record for both files
// write the particles from a particles arrray to the out file 
float modify_singlet(FILE *fpi,FILE *fpo, int np, particle *pos)
{
   char string[180];
   int npa=0; // number of particles = nRows
   int nc=0;
   double t=0.0;
   fgets(string,180,fpi);  // start header
   fprintf(fpo,"%s",string);
   fgets(string,180,fpi);  // header info
   fprintf(fpo,"%s",string);
   sscanf(string+54,"%d, nColumns=%d, time=%lf",&npa,&nc,&t); 
   fgets(string,180,fpi);  // end header
   fprintf(fpo,"%s",string);
   fgets(string,180,fpi);  // here starts the data record
   fprintf(fpo,"%s",string);
   float xx[14];
   int j;
   char k[3];
   for(int i=0;i<np;i++){
     fread(&j,sizeof(xx[0]),1,fpi); // particle number
     fwrite(&j,sizeof(xx[0]),1,fpo); // particle number

     fread(xx,sizeof(xx[0]),14,fpi); // particle data

     xx[0] = pos[i].x ; xx[1] = pos[i].y ; xx[2] = pos[i].z ;// particle position
     xx[3] = pos[i].vx; xx[4] = pos[i].vy; xx[5] = pos[i].vz;// velocity 
     xx[6] = pos[i].ax; xx[7] = pos[i].ay; xx[8] = pos[i].az;// acceleration
     xx[9] = pos[i].m; // mass
     xx[10] =pos[i].r; // radius
     xx[11] =pos[i].pressure;
     xx[12] =pos[i].density; 
     xx[13] =pos[i].energy;

     fwrite(xx,sizeof(xx[0]),14,fpo); // particle data but given here not copied

     fread(k,sizeof(k[0]),1,fpi); // single extra character here
     fwrite(k,sizeof(k[0]),1,fpo); // single extra character here

   }
   fgets(string,180,fpi); // ends the record
   fprintf(fpo,"%s",string);
   int ff = feof(fpi); // end of file?
   if (ff==0) return t;
   else return -1.0;  // returns -1 if end of file has been reached
}


