

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include"subs.h"
#include"dsyevc3.h"


// compute strain images from a part file

#define NPHI 36 // grid in polar coordinates for making strain maps
#define NTHETA 12
#define THETAMIN -M_PI/2.0

// neighborhood pancake info
#define RAD0  1.0   // radius for center of initial neighborhoods
#define DRAD0  0.25  // radius distance to center
#define RADMIN  0.9  // points must be outside this radius  

#define DO_NSNAPS 100 // how many snapshots to do
#define DSNAP_SPACING 10 // first snap and spacing to do
int main(int argc, char* argv[]){
   
   char filename[30],ofilenameroot[30];
//   strcpy(filename,"planetTidal.part");
   if (argc<3){ 
     printf(" args: filename phishift(degrees)\n");
     exit(0);
   }
   strcpy(filename, argv[1]);
   double phishift;
   sscanf(argv[2],"%lf",&phishift);
   phishift *= M_PI/180.0;

   int np = findnp(filename); // get the number of particles
   printf("# np=%d\n",np);
   printf("# sizescale=%.3f\n",1.0/pow(np,1.0/3.0));
   printf("# nsurface=%.3f\n",4.0*pow(np,2.0/3.0));
   float tmax,dt;
   int nsnaps = getnsnaps(filename,np,&dt,&tmax); // get time info
   printf("# number of snaps = %d maxtime =%f dt =%.2f\n",nsnaps,tmax,dt);

   char *pch;
   strcpy(ofilenameroot,filename); // outfilename setup
   pch=strchr(ofilenameroot,'.');
   pch[0] = '\0';

// set up dimensions of polar grid, grid in radians
   double dphi = 2.0*M_PI/(double)NPHI; // spacing for position grid
   double dtheta = M_PI/(double)NTHETA;
   double degfac = 180.0/M_PI;
   int nphi = NPHI+1; // dimensions of arrays
   int ntheta = NTHETA+1;

   particle *snap_pos_init; snap_pos_init =new particle[np];  // snap storage
   particle *snap_pos_t; snap_pos_t =new particle[np];  // snap storage
   int *nei_list; nei_list = new int[np]; // neighborlist storage
   double *strain_p_arr; strain_p_arr = new double[nphi*ntheta]; // strain storage
   double *strain_n_arr; strain_n_arr = new double[nphi*ntheta]; // strain storage
   double *ang_p_arr;  ang_p_arr = new double[nphi*ntheta]; // angle  storage
   double *ang_n_arr;  ang_n_arr = new double[nphi*ntheta]; // angle  storage
   double *rm_arr;  rm_arr = new double[nphi*ntheta]; // mean radius storage
// maxima for whole timestep
   double *strain_p_marr; strain_p_marr = new double[nphi*ntheta]; // strain storage
   double *strain_n_marr; strain_n_marr = new double[nphi*ntheta]; // strain storage
   double *ang_p_marr;  ang_p_marr = new double[nphi*ntheta]; // angle  storage
   double *ang_n_marr;  ang_n_marr = new double[nphi*ntheta]; // angle  storage
   for(int i=0;i<nphi*ntheta;i++){
      strain_p_marr[i] = 0.0; 
      strain_n_marr[i] = 0.0; 
   }

// load initial snap shot, assume we are in body frame and com has been
// subtracted aldready (ignoring last particle!)
   int i_s=0; // first snapshot
   float time = readpossnap(filename,np, i_s, snap_pos_init); // read it in

// loop here for timesteps
   int dsnap =DSNAP_SPACING; // spacing for snaps
   for(int i_snap=dsnap; i_snap <= DO_NSNAPS; i_snap += dsnap){
   //  i_snap is  which snapshot
   // load snap shot at another time
      time = readpossnap(filename,np, i_snap, snap_pos_t); // read it in
 
      char ofilename[40]; // outfilename 
      strcpy(ofilename,ofilenameroot); // make an outfile name
      strcat(ofilename,"_s"); 
      char junks[20];
      if (i_snap < 1000) strcat(ofilename,"0"); 
      if (i_snap < 100) strcat(ofilename,"0"); 
      if (i_snap < 10) strcat(ofilename,"0"); 
      sprintf(junks,"%d.dat",i_snap);
      strcat(ofilename,junks); 
      printf("%s\n",ofilename);

      FILE *fpo;
      fpo = fopen(ofilename,"w");

// loop for a grid of positions
   for(int i=0;i<nphi;i++){
   for(int j=0;j<ntheta;j++){
       double phi = (double)i*dphi + phishift; // grid in radians
       double theta = THETAMIN + (double)j*dtheta;
       float x0 = RAD0*cos(theta)*cos(phi);
       float y0 = RAD0*cos(theta)*sin(phi);
       float z0 = RAD0*sin(theta);

       // center most particle
       int centerone = nei1(snap_pos_init, np, x0, y0, z0);

       // get a neighborhood in the initial snap shot
       int n_nei = num_nei(snap_pos_init, np, DRAD0, 
                  RADMIN, x0, y0, z0, nei_list);
       if ((i+j==0)&&(i_snap == dsnap)) printf("# n_nei=%d \n",n_nei);

       double dthetadphi_p,strain_p; // positive
       maxdl(snap_pos_init, snap_pos_t, n_nei, nei_list, 
           DRAD0/2.0, centerone, 1, &strain_p, &dthetadphi_p);
       double dthetadphi_n,strain_n; // negative 
       maxdl(snap_pos_init, snap_pos_t, n_nei, nei_list, 
           DRAD0/2.0, centerone,-1, &strain_n, &dthetadphi_n);

       double rm = r_mean(snap_pos_t, n_nei, nei_list); // mean radius
       int index = i + j*nphi;
       rm_arr[index] = rm;

       strain_p_arr[index] = strain_p;
       ang_p_arr[index] = dthetadphi_p;
       strain_n_arr[index] = strain_n;
       ang_n_arr[index] = dthetadphi_n;
   }}
   for(int i=0;i<nphi;i++){ // print max strains out to file in gnuplot image format
     for(int j=0;j<ntheta;j++){
        double phi = (double)i*dphi;
        double theta = THETAMIN + (double)j*dtheta;
        int index = i + j*nphi;
        fprintf(fpo,"%8.0f %8.0f ",phi*degfac, theta*degfac);
        fprintf(fpo,"%11.5f %11.5f %11.5f %11.5f ",
             strain_p_arr[index], ang_p_arr[index], 
             strain_n_arr[index], ang_n_arr[index]);
        fprintf(fpo,"%11.5f ",rm_arr[index]);
        fprintf(fpo,"\n");
     }
     fprintf(fpo,"\n");
   }
   fclose(fpo);  // close file

// keep track of maximum strains during entire timeseries
   for(int i=0;i<nphi*ntheta;i++){
     if (strain_p_arr[i] > strain_p_marr[i]) {
                strain_p_marr[i] = strain_p_arr[i]; 
                ang_p_marr[i] = ang_p_arr[i]; 
     }
     if (strain_n_arr[i] < strain_n_marr[i]) {
                strain_n_marr[i] = strain_n_arr[i]; 
                ang_n_marr[i] = ang_n_arr[i]; 
     }
   }
   }// end of snap forloop

// make a file name for the maximum of strains over all timesteps
   char ofilename[40]; // outfilename
   strcpy(ofilename,ofilenameroot); // make an outfile name
   strcat(ofilename,"_m");
   char junks[20];
   sprintf(junks,".dat");
   strcat(ofilename,junks);
   printf("%s\n",ofilename);
   FILE *fpo;
   fpo = fopen(ofilename,"w");
   for(int i=0;i<nphi;i++){ // print max strains out to file in gnuplot image format
     for(int j=0;j<ntheta;j++){
        double phi = (double)i*dphi;
        double theta = THETAMIN + (double)j*dtheta;
        int index = i + j*nphi;
        fprintf(fpo,"%8.0f %8.0f ",phi*degfac, theta*degfac);
        fprintf(fpo,"%11.5f %11.5f %11.5f %11.5f ",
             strain_p_marr[index], ang_p_marr[index], 
             strain_n_marr[index], ang_n_marr[index]);
        fprintf(fpo,"\n");
     }
     fprintf(fpo,"\n");
   }

   fclose(fpo);  // close file



// cleanup
   delete[] nei_list;
   delete[] snap_pos_init;
   delete[] snap_pos_t;
   delete[] strain_p_arr;
   delete[] strain_n_arr;
   delete[] ang_p_arr;
   delete[] ang_n_arr;
   delete[] rm_arr;
   delete[] strain_p_marr;
   delete[] strain_n_marr;
   delete[] ang_p_marr;
   delete[] ang_n_marr;
}


